import asyncio
from datetime import datetime, timedelta
from telegram import Bot
from telegram.ext import Application, CommandHandler, MessageHandler, filters
import aiohttp
import os
import time
import json

# Bot token
BOT_TOKEN = "7616747725:AAHZuDi066QxvK9IbzGfZJWYe897-k-ibhk"

# Path to keyword lists JSON file
KEYWORD_LIST_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'keyword_lists.json')

# Configuration for multiple sources
# Only monitor requests.txt for now
SOURCE_CONFIGS = {
    "https://magnetic.website/Requests/requests.txt": {
        "header": "🎯 Titan addon Information"
    }
}

# Extract sources for easier iteration
MONITOR_SOURCES = list(SOURCE_CONFIGS.keys())

# File to store last state
STATE_FILE = "bot_state.json"

# Track last modification time and content for each source
source_states = {source: {"last_modified": 0, "last_content": ""} for source in MONITOR_SOURCES}

# Track keyword list file modification time
keyword_list_last_modified = 0

# Dictionaries to store user data
new_users = {}
user_warnings = {}

# Keyword lists and responses
keyword_data = {
    "check_list1": {"triggers": [], "response": ""},
    "check_list2": {"triggers": [], "response": ""},
    "check_list3": {"triggers": [], "response": ""}
}

# Initialize target_chat_id globally
target_chat_id = None

def load_state():
    """Load the last known state for each source from file."""
    global source_states
    try:
        if os.path.exists(STATE_FILE):
            with open(STATE_FILE, 'r', encoding='utf-8') as f:
                state = json.load(f)
                for source in MONITOR_SOURCES:
                    if source in state:
                        source_states[source]["last_modified"] = state[source].get("last_modified", 0)
                        source_states[source]["last_content"] = state[source].get("last_content", "")
    except Exception as e:
        print(f"Error loading state: {e}")

def save_state():
    """Save the current state for each source to file."""
    try:
        state = {
            source: {
                "last_modified": source_states[source]["last_modified"],
                "last_content": source_states[source]["last_content"]
            }
            for source in MONITOR_SOURCES
        }
        with open(STATE_FILE, 'w', encoding='utf-8') as f:
            json.dump(state, f)
    except Exception as e:
        print(f"Error saving state: {e}")

def load_keyword_lists():
    """Load keyword lists and responses from local JSON file."""
    global keyword_data
    try:
        with open(KEYWORD_LIST_PATH, 'r') as file:
            keyword_data = json.load(file)
            print("Keyword lists and responses loaded successfully.")
    except Exception as e:
        print(f"Error loading keyword lists: {e}")

async def handle_message(update, context):
    """Handle incoming messages and check for keyword triggers."""
    global target_chat_id
    
    if target_chat_id is None:
        target_chat_id = update.message.chat.id
        print(f"Target chat ID set to: {target_chat_id}")
        
    if not update.message or not update.message.text:
        return
        
    text = update.message.text.lower()
    user_id = update.message.from_user.id
    first_name = update.message.from_user.first_name
    chat_id = update.message.chat.id
    
    for list_name, data in keyword_data.items():
        # Skip blocked_content for chat messages
        if list_name == "blocked_content":
            continue
            
        if any(trigger in text for trigger in data.get("triggers", [])):
            # Send response
            if data.get("response"):
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=f"{first_name}\n{data['response']}"
                )
            
            # Send image if available
            if data.get("image_url"):
                try:
                    await context.bot.send_photo(
                        chat_id=chat_id,
                        photo=data["image_url"]
                    )
                except Exception as e:
                    print(f"Error sending image: {e}")
            break

async def monitor_file(bot):
    """Monitor multiple sources (files or URLs) for changes and send updates to Telegram."""
    global target_chat_id, source_states

    print(f"Monitoring sources: {', '.join(MONITOR_SOURCES)}")

    # Wait until we have a target chat ID
    while target_chat_id is None:
        await asyncio.sleep(1)
        print("Waiting for target chat ID...")
        continue

    print(f"Started monitoring with target chat ID: {target_chat_id}")
    
    # Start with clean state
    global source_states
    source_states = {source: {"last_modified": 0, "last_content": ""} for source in MONITOR_SOURCES}
    save_state()
    print("Starting with clean state")

    while True:
        try:
            for source in MONITOR_SOURCES:
                print(f"\nChecking {source}")
                config = SOURCE_CONFIGS[source]
                header = config["header"]

                # Get last content for comparison
                last_content = source_states[source]["last_content"]

                try:
                    async with aiohttp.ClientSession() as session:
                        async with session.get(source, timeout=30) as response:
                            if response.status == 200:
                                current_content = (await response.text()).strip()
                                
                                # Only process if content has changed
                                if current_content == last_content:
                                    print(f"Content unchanged for {source}, skipping")
                                    continue
                                    
                                print(f"New content found for {source}")
                            elif response.status == 404:
                                print(f"Source not found: {source}")
                                continue
                            else:
                                print(f"Failed to fetch {source}. Status: {response.status}")
                                continue
                except Exception as e:
                    print(f"Error fetching {source}: {e}")
                    continue

                # Skip empty content
                if not current_content:
                    print("No content received")
                    continue

                # Process requests.txt content
                if source == "https://magnetic.website/Requests/requests.txt":
                    # Find new content by comparing with last state
                    new_lines = []
                    current_lines = current_content.strip().split('\n')
                    last_lines = last_content.strip().split('\n') if last_content else []
                    
                    # Get only the new lines that weren't in the last content
                    new_lines = [line.strip() for line in current_lines if line.strip() and line.strip() not in last_lines]
                    
                    if new_lines:
                        print(f"Found {len(new_lines)} new lines")
                        processed_lines = []
                        
                        # Process only new lines
                        for line in new_lines:
                            is_blocked = False
                            if "blocked_content" in keyword_data:
                                for trigger in keyword_data["blocked_content"]["triggers"]:
                                    if trigger.lower() in line.lower():
                                        is_blocked = True
                                        print(f"Found blocked content: {trigger}")
                                        try:
                                            await bot.send_message(
                                                chat_id=target_chat_id,
                                                text=keyword_data["blocked_content"]["response"]
                                            )
                                        except Exception as e:
                                            print(f"Error sending blocked message: {e}")
                                        break
                            
                            if not is_blocked:
                                processed_lines.append(line)
                        
                        # Send only new non-blocked lines
                        if processed_lines:
                            try:
                                message = f"{header}:\n\n" + "\n".join(processed_lines)
                                await bot.send_message(
                                    chat_id=target_chat_id,
                                    text=message
                                )
                                print(f"Sent {len(processed_lines)} new lines")
                            except Exception as e:
                                print(f"Error sending to Telegram: {e}")
                    else:
                        print("No new content to process")

                # For other files, only send if content has changed
                elif source != "https://magnetic.website/Requests/requests.txt" and current_content != last_content:
                    try:
                        await bot.send_message(
                            chat_id=target_chat_id,
                            text=f"{header}:\n\n{current_content}"
                        )
                        print(f"Sent updated content for {source}")
                    except Exception as e:
                        print(f"Error sending message: {e}")

                # Always update state for all sources
                source_states[source]["last_content"] = current_content
                source_states[source]["last_modified"] = str(time.time())
                save_state()
                print(f"Updated state for {source}")

                # Add delay between sources
                await asyncio.sleep(1)

            # Save state after processing all sources
            save_state()
            print("State saved, waiting before next check")
            
            # Sleep between checks
            await asyncio.sleep(10)  # 10 second delay between checks

        except Exception as e:
            print(f"Error in monitor loop: {e}")
            print(f"Error details: {str(e)}")
            await asyncio.sleep(5)

async def run_bot():
    """Run the bot and set up all handlers."""
    application = Application.builder().token(BOT_TOKEN).build()
    
    # Load keywords
    load_keyword_lists()
    print("Keyword lists loaded")
    
    # Add handlers
    application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, handle_message))
    
    # Start the bot
    bot = Bot(token=BOT_TOKEN)
    
    try:
        print("Starting bot tasks...")
        await application.initialize()
        await application.start()
        await application.updater.start_polling()
        
        print("Starting file monitor...")
        monitor_task = asyncio.create_task(monitor_file(bot))
        
        # Keep the bot running
        while True:
            await asyncio.sleep(1)
            
    except Exception as e:
        print(f"Error in bot: {e}")
    finally:
        print("Shutting down...")
        await application.stop()
        await application.shutdown()

def main():
    """Entry point for running the bot."""
    # Load initial state
    load_state()
    
    print("Starting bot")
    asyncio.run(run_bot())

if __name__ == "__main__":
    try:
        # Start fresh each time
        if os.path.exists(STATE_FILE):
            os.remove(STATE_FILE)
        print("Starting fresh")
        
        main()
    except KeyboardInterrupt:
        print("\nBot stopped by user")
    except Exception as e:
        print(f"Fatal error: {e}")